@@ -6,9 +6,7 @@
     :license: MIT, see LICENSE for more details.
 """
 import datetime
-import itertools
 import logging
-import random
 import socket
 import time
 import warnings
@@ -58,6 +56,7 @@ class VSManager(utils.IdentifierMixin, object):
         else:
             self.ordering_manager = ordering_manager
 
+    @retry(logger=LOGGER)
     def list_instances(self, hourly=True, monthly=True, tags=None, cpus=None,
                        memory=None, hostname=None, domain=None,
                        local_disk=None, datacenter=None, nic_speed=None,
@@ -161,6 +160,7 @@ class VSManager(utils.IdentifierMixin, object):
         func = getattr(self.account, call)
         return func(**kwargs)
 
+    @retry(logger=LOGGER)
     def get_instance(self, instance_id, **kwargs):
         """Get details about a virtual server instance.
 
@@ -235,6 +235,7 @@ class VSManager(utils.IdentifierMixin, object):
 
         return self.guest.getObject(id=instance_id, **kwargs)
 
+    @retry(logger=LOGGER)
     def get_create_options(self):
         """Retrieves the available options for creating a VS.
 
@@ -271,7 +272,7 @@ class VSManager(utils.IdentifierMixin, object):
         :param string post_url: The URI of the post-install script to run
                                 after reload
         :param list ssh_keys: The SSH keys to add to the root user
-        :param int image_id: The ID of the image to load onto the server
+        :param int image_id: The GUID of the image to load onto the server
 
         .. warning::
             This will reformat the primary drive.
@@ -306,7 +307,7 @@ class VSManager(utils.IdentifierMixin, object):
             dedicated=False, public_vlan=None, private_vlan=None,
             userdata=None, nic_speed=None, disks=None, post_uri=None,
             private=False, ssh_keys=None, public_security_groups=None,
-            private_security_groups=None, **kwargs):
+            private_security_groups=None, boot_mode=None, **kwargs):
         """Returns a dict appropriate to pass into Virtual_Guest::createObject
 
             See :func:`create_instance` for a list of available options.
@@ -338,11 +339,14 @@ class VSManager(utils.IdentifierMixin, object):
             "hostname": hostname,
             "domain": domain,
             "localDiskFlag": local_disk,
-            "hourlyBillingFlag": hourly
+            "hourlyBillingFlag": hourly,
+            "supplementalCreateObjectOptions": {
+                "bootMode": boot_mode
+            }
         }
 
         if flavor:
-            data["supplementalCreateObjectOptions"] = {"flavorKeyName": flavor}
+            data["supplementalCreateObjectOptions"]["flavorKeyName"] = flavor
 
         if dedicated and not host_id:
             data["dedicatedAccountHostOnlyFlag"] = dedicated
@@ -411,6 +415,7 @@ class VSManager(utils.IdentifierMixin, object):
 
         return data
 
+    @retry(logger=LOGGER)
     def wait_for_transaction(self, instance_id, limit, delay=10):
         """Waits on a VS transaction for the specified amount of time.
 
@@ -424,7 +429,7 @@ class VSManager(utils.IdentifierMixin, object):
 
         return self.wait_for_ready(instance_id, limit, delay=delay, pending=True)
 
-    def wait_for_ready(self, instance_id, limit, delay=10, pending=False):
+    def wait_for_ready(self, instance_id, limit=3600, delay=10, pending=False):
         """Determine if a VS is ready and available.
 
         In some cases though, that can mean that no transactions are running.
@@ -435,7 +440,7 @@ class VSManager(utils.IdentifierMixin, object):
         cancellations.
 
         :param int instance_id: The instance ID with the pending transaction
-        :param int limit: The maximum amount of time to wait.
+        :param int limit: The maximum amount of seconds to wait.
         :param int delay: The number of seconds to sleep before checks. Defaults to 10.
         :param bool pending: Wait for pending transactions not related to
                              provisioning or reloads such as monitoring.
@@ -445,43 +450,22 @@ class VSManager(utils.IdentifierMixin, object):
             # Will return once vsi 12345 is ready, or after 10 checks
             ready = mgr.wait_for_ready(12345, 10)
         """
-        until = time.time() + limit
-        for new_instance in itertools.repeat(instance_id):
-            mask = """id,
-                      lastOperatingSystemReload.id,
-                      activeTransaction.id,provisionDate"""
-            try:
-                instance = self.get_instance(new_instance, mask=mask)
-                last_reload = utils.lookup(instance, 'lastOperatingSystemReload', 'id')
-                active_transaction = utils.lookup(instance, 'activeTransaction', 'id')
-
-                reloading = all((
-                    active_transaction,
-                    last_reload,
-                    last_reload == active_transaction,
-                ))
-
-                # only check for outstanding transactions if requested
-                outstanding = False
-                if pending:
-                    outstanding = active_transaction
-
-                # return True if the instance has finished provisioning
-                # and isn't currently reloading the OS.
-                if all([instance.get('provisionDate'),
-                        not reloading,
-                        not outstanding]):
-                    return True
-                LOGGER.info("%s not ready.", str(instance_id))
-            except exceptions.SoftLayerAPIError as exception:
-                delay = (delay * 2) + random.randint(0, 9)
-                LOGGER.info('Exception: %s', str(exception))
+        now = time.time()
+        until = now + limit
+        mask = "mask[id, lastOperatingSystemReload[id], activeTransaction, provisionDate]"
 
+        while now <= until:
+            instance = self.get_instance(instance_id, mask=mask)
+            if utils.is_ready(instance, pending):
+                return True
+            transaction = utils.lookup(instance, 'activeTransaction', 'transactionStatus', 'friendlyName')
+            snooze = min(delay, until - now)
+            LOGGER.info("%s - %d not ready. Auto retry in %ds", transaction, instance_id, snooze)
+            time.sleep(snooze)
             now = time.time()
-            if now >= until:
-                return False
-            LOGGER.info('Auto retry in %s seconds', str(min(delay, until - now)))
-            time.sleep(min(delay, until - now))
+
+        LOGGER.info("Waiting for %d expired.", instance_id)
+        return False
 
     def verify_create_instance(self, **kwargs):
         """Verifies an instance creation command.
@@ -553,7 +537,7 @@ class VSManager(utils.IdentifierMixin, object):
         :param bool local_disk: Flag to indicate if this should be a local disk (default) or a SAN disk.
         :param string datacenter: The short name of the data center in which the VS should reside.
         :param string os_code: The operating system to use. Cannot be specified  if image_id is specified.
-        :param int image_id: The ID of the image to load onto the server. Cannot be specified if os_code is specified.
+        :param int image_id: The GUID of the image to load onto the server. Cannot be specified if os_code is specified.
         :param bool dedicated: Flag to indicate if this should be housed on adedicated or shared host (default).
                                This will incur a fee on your account.
         :param int public_vlan: The ID of the public VLAN on which you want this VS placed.
@@ -576,7 +560,7 @@ class VSManager(utils.IdentifierMixin, object):
             self.set_tags(tags, guest_id=inst['id'])
         return inst
 
-    @retry(exceptions.SoftLayerAPIError, logger=LOGGER)
+    @retry(logger=LOGGER)
     def set_tags(self, tags, guest_id):
         """Sets tags on a guest with a retry decorator
 
@@ -670,7 +654,7 @@ class VSManager(utils.IdentifierMixin, object):
         results = self.list_instances(hostname=hostname, mask="id")
         return [result['id'] for result in results]
 
-    def _get_ids_from_ip(self, ip_address):
+    def _get_ids_from_ip(self, ip_address):  # pylint: disable=inconsistent-return-statements
         """List VS ids which match the given ip address."""
         try:
             # Does it look like an ip address?
@@ -871,9 +855,6 @@ class VSManager(utils.IdentifierMixin, object):
         package_keyname = "CLOUD_SERVER"
         package = self.ordering_manager.get_package_by_key(package_keyname)
 
-        if package is None:
-            raise ValueError("No package found for key: " + package_keyname)
-
         package_service = self.client['Product_Package']
         return package_service.getItems(id=package['id'], mask=mask)
 
@@ -893,8 +874,8 @@ class VSManager(utils.IdentifierMixin, object):
         mask = "mask[%s]" % ','.join(mask)
         return self.guest.getUpgradeItemPrices(include_downgrade_options, id=instance_id, mask=mask)
 
-    def _get_price_id_for_upgrade_option(self, upgrade_prices, option, value,
-                                         public=True):
+    # pylint: disable=inconsistent-return-statements
+    def _get_price_id_for_upgrade_option(self, upgrade_prices, option, value, public=True):
         """Find the price id for the option and value to upgrade. This
 
         :param list upgrade_prices: Contains all the prices related to a VS upgrade
@@ -934,8 +915,8 @@ class VSManager(utils.IdentifierMixin, object):
                 else:
                     return price.get('id')
 
-    def _get_price_id_for_upgrade(self, package_items, option, value,
-                                  public=True):
+    # pylint: disable=inconsistent-return-statements
+    def _get_price_id_for_upgrade(self, package_items, option, value, public=True):
         """Find the price id for the option and value to upgrade.
 
         Deprecated in favor of _get_price_id_for_upgrade_option()
